@@ -6,7 +6,7 @@ IMDb's data for mobile systems.
 the imdb.IMDb function will return an instance of this class when
 called with the 'accessSystem' argument set to "mobile".
 
-Copyright 2005-2007 Davide Alberani <da@erlug.linux.it>
+Copyright 2005-2010 Davide Alberani <da@erlug.linux.it>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -24,8 +24,8 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """
 
 import re
+import logging
 from urllib import unquote
-from types import ListType, TupleType
 
 from imdb import imdbURL_movie_main, imdbURL_person_main, imdbURL_character_main
 from imdb.Movie import Movie
@@ -49,6 +49,9 @@ re_unhtmlsub = re_unhtml.sub
 # imdb person or movie ids.
 re_imdbID = re.compile(r'(?<=nm|tt|ch)([0-9]{7})\b')
 
+# movie AKAs.
+re_makas = re.compile('(<p class="find-aka">.*?</p>)')
+
 
 def _unHtml(s):
     """Return a string without tags and no multiple spaces."""
@@ -89,14 +92,15 @@ def _getTagsWith(s, cont, toClosure=False, maxRes=None):
     return lres
 
 
-def _findBetween(s, begins, ends, beginindx=0, maxRes=None):
+def _findBetween(s, begins, ends, beginindx=0, maxRes=None, lres=None):
     """Return the list of strings from the 's' string which are included
     between the 'begins' and 'ends' strings."""
-    lres = []
+    if lres is None:
+        lres = []
     bi = s.find(begins, beginindx)
     if bi != -1:
         lbegins = len(begins)
-        if isinstance(ends, (ListType, TupleType)):
+        if isinstance(ends, (list, tuple)):
             eset = [s.find(end, bi+lbegins) for end in ends]
             eset[:] = [x for x in eset if x != -1]
             if not eset: ei = -1
@@ -107,7 +111,8 @@ def _findBetween(s, begins, ends, beginindx=0, maxRes=None):
             match = s[bi+lbegins:ei]
             lres.append(match)
             if maxRes is not None and len(lres) >= maxRes: return lres
-            lres += _findBetween(s, begins, ends, ei)
+            _findBetween(s, begins, ends, beginindx=ei, maxRes=maxRes,
+                        lres=lres)
     return lres
 
 
@@ -116,6 +121,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
     mobile terminals."""
 
     accessSystem = 'mobile'
+    _mobile_logger = logging.getLogger('imdbpy.parser.mobile')
 
     def __init__(self, isThin=1, *arguments, **keywords):
         self.accessSystem = 'mobile'
@@ -177,28 +183,54 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         cont = subXMLRefs(self._get_search_content('tt', title, results))
         title = _findBetween(cont, '<title>', '</title>', maxRes=1)
         res = []
-        if not title: return res
+        if not title:
+            self._mobile_logger.error('no title tag searching for movie %s',
+                                    title)
+            return res
         tl = title[0].lower()
         if not tl.startswith('imdb title'):
-            # XXX: a direct hit!
+            # a direct hit!
             title = _unHtml(title[0])
-            midtag = _getTagsWith(cont, 'name="arg"', maxRes=1)
-            if not midtag: midtag = _getTagsWith(cont, 'name="auto"', maxRes=1)
             mid = None
+            midtag = _getTagsWith(cont, 'rel="canonical"', maxRes=1)
             if midtag:
-                mid = _findBetween(midtag[0], 'value="', '"', maxRes=1)
-                if mid and not mid[0].isdigit():
-                    mid = re_imdbID.findall(mid[0])
-            if not (mid and title): return res
-            res[:] = [(str(mid[0]), analyze_title(title, canonical=1))]
+                mid = _findBetween(midtag[0], '/title/tt', '/', maxRes=1)
+            if not (mid and title):
+                self._mobile_logger.error('no direct hit title/movieID for' \
+                                            ' title %s', title)
+                return res
+            if cont.find('<span class="tv-extra">TV mini-series</span>') != -1:
+                title += ' (mini)'
+            res[:] = [(str(mid[0]), analyze_title(title))]
         else:
-            lis = _findBetween(cont, 'td valign="top">',
-                                ['</td>', '<br>', '<br/>'])
+            # XXX: this results*3 prevents some recursion errors, but...
+            #      it's not exactly understandable (i.e.: why 'results' is
+            #      not enough to get all the results?)
+            lis = _findBetween(cont, 'td valign="top">', '</td>',
+                                maxRes=results*3)
             for li in lis:
+                akas = re_makas.findall(li)
+                for idx, aka in enumerate(akas):
+                    aka = aka.replace('" - ', '::', 1)
+                    aka = _unHtml(aka)
+                    if aka.startswith('aka "'):
+                        aka = aka[5:].strip()
+                    if aka[-1] == '"':
+                        aka = aka[:-1]
+                    akas[idx] = aka
                 imdbid = re_imdbID.findall(li)
+                li = re_makas.sub('', li)
                 mtitle = _unHtml(li)
-                if not (imdbid and mtitle): continue
-                res.append((str(imdbid[0]), analyze_title(mtitle, canonical=1)))
+                if not (imdbid and mtitle):
+                    self._mobile_logger.debug('no title/movieID parsing' \
+                                            ' %s searching for title %s', li,
+                                            title)
+                    continue
+                mtitle = mtitle.replace('(TV mini-series)', '(mini)')
+                resd = analyze_title(mtitle)
+                if akas:
+                    resd['akas'] = akas
+                res.append((str(imdbid[0]), resd))
         return res
 
     def get_movie_main(self, movieID):
@@ -207,14 +239,16 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         if not title:
             raise IMDbDataAccessError, 'unable to get movieID "%s"' % movieID
         title = _unHtml(title[0])
-        d = analyze_title(title, canonical=1)
+        if cont.find('<span class="tv-extra">TV mini-series</span>') != -1:
+            title += ' (mini)'
+        d = analyze_title(title)
         kind = d.get('kind')
         tv_series = _findBetween(cont, 'TV Series:</h5>', '</a>', maxRes=1)
         if tv_series: mid = re_imdbID.findall(tv_series[0])
         else: mid = None
         if tv_series and mid:
             s_title = _unHtml(tv_series[0])
-            s_data = analyze_title(s_title, canonical=1)
+            s_data = analyze_title(s_title)
             m = Movie(movieID=str(mid[0]), data=s_data,
                         accessSystem=self.accessSystem,
                         modFunct=self._defModFunct)
@@ -223,7 +257,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         if kind in ('tv series', 'tv mini series'):
             years = _findBetween(cont, '<h1>', '</h1>', maxRes=1)
             if years:
-                years[:] = _findBetween(years[0], 'TV-Series', '</span>',
+                years[:] = _findBetween(years[0], 'TV series', '</span>',
                                         maxRes=1)
                 if years:
                     d['series years'] = years[0].strip()
@@ -233,7 +267,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
             air_date = air_date[0]
             vi = air_date.find('(')
             if vi != -1:
-                date = air_date[:vi].strip()
+                date = _unHtml(air_date[:vi]).strip()
                 if date != '????':
                     d['original air date'] = date
                 air_date = air_date[vi:]
@@ -273,6 +307,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
             if not creator:
                 # They change 'Created by' to 'Creator' and viceversa
                 # from time to time...
+                # XXX: is 'Creators' also used?
                 creator = _findBetween(cont, 'Creator:</h5>',
                                         ('class="tn15more"', '</div>',
                                         '<br/> <br/>'), maxRes=1)
@@ -295,8 +330,10 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
             cvurl = _findBetween(cvurl[0], 'src="', '"', maxRes=1)
             if cvurl: d['cover url'] = cvurl[0]
         genres = _findBetween(cont, 'href="/Sections/Genres/', '/')
-        if genres: d['genres'] = genres
-        ur = _findBetween(cont, 'User Rating:</b>', ' votes</a>)', maxRes=1)
+        if genres:
+            d['genres'] = list(set(genres))
+        ur = _findBetween(cont, '<div class="starbar-meta">', '</div>',
+                            maxRes=1)
         if ur:
             rat = _findBetween(ur[0], '<b>', '</b>', maxRes=1)
             if rat:
@@ -307,14 +344,15 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
                         rat = float(rat.strip())
                         d['rating'] = rat
                     except ValueError:
-                        pass
-            vi = ur[0].rfind('(')
-            if vi != -1 and ur[0][vi:].find('await') == -1:
+                        self._mobile_logger.warn('wrong rating: %s', rat)
+            vi = ur[0].rfind('tn15more">')
+            if vi != -1 and ur[0][vi+10:].find('await') == -1:
                 try:
-                    votes = int(_unHtml(ur[0][vi+1:]).strip().replace(',', ''))
+                    votes = _unHtml(ur[0][vi+10:]).replace('votes', '').strip()
+                    votes = int(votes.replace(',', ''))
                     d['votes'] = votes
                 except ValueError:
-                    pass
+                    self._mobile_logger.warn('wrong votes: %s', ur)
         top250 = _findBetween(cont, 'href="/chart/top?', '</a>', maxRes=1)
         if top250:
             fn = top250[0].rfind('#')
@@ -323,7 +361,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
                     td = int(top250[0][fn+1:])
                     d['top 250 rank'] = td
                 except ValueError:
-                    pass
+                    self._mobile_logger.warn('wrong top250: %s', top250)
         castdata = _findBetween(cont, 'Cast overview', '</table>', maxRes=1)
         if not castdata:
             castdata = _findBetween(cont, 'Credited cast', '</table>', maxRes=1)
@@ -355,8 +393,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         if akas:
             # For some reason, here <br> is still used in place of <br/>.
             akas[:] = [x for x in akas[0].split('<br>') if x.strip()]
-            akas = [_unHtml(x).replace(' (','::(', 1).replace(' [','::[')
-                    for x in akas]
+            akas = [_unHtml(x).replace(' (','::(', 1) for x in akas]
             if 'more' in akas: akas.remove('more')
             d['akas'] = akas
         mpaa = _findBetween(cont, 'MPAA</a>:', '</div>', maxRes=1)
@@ -365,8 +402,8 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         if runtimes:
             runtimes = runtimes[0]
             runtimes = [x.strip().replace(' min', '').replace(' (', '::(', 1)
-                    for x in runtimes.split('/')]
-            d['runtimes'] = runtimes
+                    for x in runtimes.split('|')]
+            d['runtimes'] = [_unHtml(x).strip() for x in runtimes]
         if kind == 'episode':
             # number of episodes.
             epsn = _findBetween(cont, 'title="Full Episode List">', '</a>',
@@ -374,43 +411,53 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
             if epsn:
                 epsn = epsn[0].replace(' Episodes', '').strip()
                 if epsn:
-                    try: epsn = int(epsn)
-                    except: pass
+                    try:
+                        epsn = int(epsn)
+                    except:
+                        self._mobile_logger.warn('wrong episodes #: %s', epsn)
                     d['number of episodes'] = epsn
         country = _findBetween(cont, 'Country:</h5>', '</div>', maxRes=1)
         if country:
-            country[:] = country[0].split(' / ')
+            country[:] = country[0].split(' | ')
             country[:] = ['<a %s' % x for x in country if x]
             country[:] = [_unHtml(x.replace(' <i>', '::')) for x in country]
             if country: d['countries'] = country
         lang = _findBetween(cont, 'Language:</h5>', '</div>', maxRes=1)
         if lang:
-            lang[:] = lang[0].split(' / ')
+            lang[:] = lang[0].split(' | ')
             lang[:] = ['<a %s' % x for x in lang if x]
             lang[:] = [_unHtml(x.replace(' <i>', '::')) for x in lang]
             if lang: d['languages'] = lang
-        col = _findBetween(cont, '"/List?color-info=', '</div>')
+        col = _findBetween(cont, '"/search/title?colors=', '</div>')
         if col:
-            col[:] = col[0].split(' / ')
+            col[:] = col[0].split(' | ')
             col[:] = ['<a %s' % x for x in col if x]
             col[:] = [_unHtml(x.replace(' <i>', '::')) for x in col]
             if col: d['color info'] = col
-        sm = _findBetween(cont, '/List?sound-mix=', '</div>', maxRes=1)
+        sm = _findBetween(cont, '/search/title?sound_mixes=', '</div>',
+                            maxRes=1)
         if sm:
-            sm[:] = sm[0].split(' / ')
+            sm[:] = sm[0].split(' | ')
             sm[:] = ['<a %s' % x for x in sm if x]
             sm[:] = [_unHtml(x.replace(' <i>', '::')) for x in sm]
             if sm: d['sound mix'] = sm
         cert = _findBetween(cont, 'Certification:</h5>', '</div>', maxRes=1)
         if cert:
-            cert[:] = cert[0].split(' / ')
+            cert[:] = cert[0].split(' | ')
             cert[:] = [_unHtml(x.replace(' <i>', '::')) for x in cert]
             if cert: d['certificates'] = cert
-        plotoutline = _findBetween(cont, 'Plot Outline:</h5>', ['<a ','</div>'],
+        plotoutline = _findBetween(cont, 'Plot:</h5>', ['<a ', '</div>'],
                                     maxRes=1)
         if plotoutline:
             plotoutline = plotoutline[0].strip()
-            if plotoutline: d['plot outline'] = plotoutline
+            plotoutline = plotoutline.rstrip('|').rstrip()
+            if plotoutline: d['plot outline'] = _unHtml(plotoutline)
+        aratio = _findBetween(cont, 'Aspect Ratio:</h5>', ['<a ', '</div>'],
+                            maxRes=1)
+        if aratio:
+            aratio = aratio[0].strip().replace(' (', '::(', 1)
+            if aratio:
+                d['aspect ratio'] = _unHtml(aratio)
         return {'data': d}
 
     def get_movie_plot(self, movieID):
@@ -422,8 +469,10 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
             wbyidx = p.rfind(' Written by ')
             if wbyidx != -1:
                 plot[i] = '%s::%s' % \
-                    (p[wbyidx+12:].rstrip().replace('{','<').replace('}','>'),
-                    p[:wbyidx].rstrip())
+                        (p[:wbyidx].rstrip(),
+                    p[wbyidx+12:].rstrip().replace('{','<').replace('}','>'))
+            else:
+                plot[i] = '%s::Anonymous' % p
         if plot: return {'data': {'plot': plot}}
         return {'data': {}}
 
@@ -434,36 +483,43 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         cont = subXMLRefs(self._get_search_content('nm', name, results))
         name = _findBetween(cont, '<title>', '</title>', maxRes=1)
         res = []
-        if not name: return res
+        if not name:
+            self._mobile_logger.warn('no title tag searching for name %s', name)
+            return res
         nl = name[0].lower()
         if not nl.startswith('imdb name'):
-            # XXX: a direct hit!
+            # a direct hit!
             name = _unHtml(name[0])
-            # Easiest way: the board link (for person who already have
-            # messages in the board).
-            pidtag = _getTagsWith(cont, '/board/nest/', maxRes=1)
+            name = name.replace('- Filmography by type' , '').strip()
             pid = None
-            if pidtag: pid = _findBetween(pidtag[0], '/name/nm', '/', maxRes=1)
+            pidtag = _getTagsWith(cont, 'rel="canonical"', maxRes=1)
+            if pidtag:
+                pid = _findBetween(pidtag[0], '/name/nm', '/', maxRes=1)
             if not (pid and name):
-                # Otherwise, the 'credited alongside' for the name,
-                # and the biography link for the personID.
-                nametag = _getTagsWith(cont, 'NAME="primary"', maxRes=1)
-                if not nametag: return res
-                nametag = _findBetween(nametag[0], 'VALUE="', '"', maxRes=1)
-                if not nametag: return res
-                name = unquote(nametag[0])
-                pid = _findBetween(cont, '/name/nm', '/bio', maxRes=1)
-                if not pid: return res
-            if not (pid and name): return res
+                self._mobile_logger.error('no direct hit name/personID for' \
+                                            ' name %s', name)
+                return res
             res[:] = [(str(pid[0]), analyze_name(name, canonical=1))]
         else:
-            lis = _findBetween(cont, 'td valign="top">',
-                                ['<small', '</td>', '<br'])
+            lis = _findBetween(cont, 'td valign="top">', '</td>',
+                                maxRes=results*3)
             for li in lis:
+                akas = _findBetween(li, '<em>"', '"</em>')
+                for sep in ['<small', '<br> aka', '<br> birth name']:
+                    sepIdx = li.find(sep)
+                    if sepIdx != -1:
+                        li = li[:sepIdx]
                 pid = re_imdbID.findall(li)
                 pname = _unHtml(li)
-                if not (pid and pname): continue
-                res.append((str(pid[0]), analyze_name(pname, canonical=1)))
+                if not (pid and pname):
+                    self._mobile_logger.debug('no name/personID parsing' \
+                                            ' %s searching for name %s', li,
+                                            name)
+                    continue
+                resd = analyze_name(pname, canonical=1)
+                if akas:
+                    resd['akas'] = akas
+                res.append((str(pid[0]), resd))
         return res
 
     def get_person_main(self, personID, _parseChr=False):
@@ -481,6 +537,9 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         name = _unHtml(name[0])
         if _parseChr:
             name = name.replace('(Character)', '').strip()
+            name = name.replace('- Filmography by type', '').strip()
+        else:
+            name = name.replace('- Filmography by', '').strip()
         r = analyze_name(name, canonical=not _parseChr)
         for dKind in ('birth', 'death'):
             date = _findBetween(s, '<h5>Date of %s:</h5>' % dKind.capitalize(),
@@ -496,7 +555,11 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         akas = _findBetween(s, 'Alternate Names:</h5>', ('</div>',
                             '<br/><br/>'), maxRes=1)
         if akas:
-            akas = _unHtml(akas[0]).split(' / ')
+            akas = akas[0]
+            if akas.find(' | ') != -1:
+                akas = _unHtml(akas).split(' | ')
+            else:
+                akas = _unHtml(akas).split(' / ')
             if akas: r['akas'] = akas
         hs = _findBetween(s, 'name="headshot"', '</a>', maxRes=1)
         if hs:
@@ -516,18 +579,21 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         for work in workkind:
             wsplit = work.split('">', 1)
             if len(wsplit) == 2:
-                ws.append((wsplit[0], wsplit[1].lower()))
+                sect = wsplit[0]
+                if '"' in sect:
+                    sect = sect[:sect.find('"')]
+                ws.append((sect, wsplit[1].lower()))
         # XXX: I think "guest appearances" are gone.
         if s.find('<a href="#guest-appearances"') != -1:
             ws.append(('guest-appearances', 'notable tv guest appearances'))
-        if _parseChr:
-            ws.append(('filmography', 'filmography'))
+        #if _parseChr:
+        #    ws.append(('filmography', 'filmography'))
         for sect, sectName in ws:
             raws = u''
             # Everything between the current section link and the end
             # of the <ol> tag.
             if _parseChr and sect == 'filmography':
-                inisect = s.find('<ul class="label">')
+                inisect = s.find('<div class="filmo">')
             else:
                 inisect = s.find('<a name="%s' % sect)
             if inisect != -1:
@@ -538,7 +604,9 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
             for m in mlist:
                 # For every movie in the current section.
                 movieID = re_imdbID.findall(m)
-                if not movieID: continue
+                if not movieID:
+                    self._mobile_logger.debug('no movieID in %s', m)
+                    continue
                 if not _parseChr:
                     chrIndx = m.find(' .... ')
                 else:
@@ -568,7 +636,9 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
                         status = _unHtml(m[stidx+3:stendidx])
                         m = m.replace(m[stidx+3:stendidx], '')
                 m = _unHtml(m)
-                if not m: continue
+                if not m:
+                    self._mobile_logger.warn('no title fo rmovieID %s', movieID)
+                    continue
                 movie = build_movie(m, movieID=movieID, status=status,
                                     roleID=chids, modFunct=self._defModFunct,
                                     accessSystem=self.accessSystem,
@@ -586,7 +656,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
                 try:
                     vtag = unquote(str(vtag[0]))
                     vtag = unicode(vtag, 'latin_1')
-                    r.update(analyze_name(vtag, canonical=0))
+                    r.update(analyze_name(vtag))
                 except UnicodeEncodeError:
                     pass
         return {'data': r, 'info sets': ('main', 'filmography')}
@@ -605,15 +675,23 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
                 spouse = spouse.replace(':: ', '::').strip()
                 if spouse: sl.append(spouse)
             if sl: d['spouse'] = sl
+        nnames = _findBetween(cont, '<h5>Nickname</h5>', ('<br/> <br/>','<h5>'),
+                                maxRes=1)
+        if nnames:
+            nnames = nnames[0]
+            if nnames:
+                nnames = [x.strip().replace(' (', '::(', 1)
+                            for x in nnames.split('<br/>')]
+                if nnames:
+                    d['nick names'] = nnames
         misc_sects = _findBetween(cont, '<h5>', '<br/>')
         misc_sects[:] = [x.split('</h5>') for x in misc_sects]
         misc_sects[:] = [x for x in misc_sects if len(x) == 2]
         for sect, data in misc_sects:
             sect = sect.lower().replace(':', '').strip()
-            if d.has_key(sect): continue
+            if d.has_key(sect) and sect != 'mini biography': continue
+            elif sect in ('spouse', 'nickname'): continue
             if sect == 'salary': sect = 'salary history'
-            elif sect == 'spouse': continue
-            elif sect == 'nickname': sect = 'nick names'
             elif sect == 'where are they now': sect = 'where now'
             elif sect == 'personal quotes': sect = 'quotes'
             data = data.replace('</p><p>', '::')
@@ -644,10 +722,15 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
                 for bio in data:
                     byidx = bio.rfind('IMDb Mini Biography By')
                     if byidx != -1:
-                        bio = u'%s::%s' % (bio[byidx+23:].lstrip(),
-                                            bio[:byidx].rstrip())
+                        bioAuth = bio[:byidx].rstrip()
+                    else:
+                        bioAuth = 'Anonymous'
+                    bio = u'%s::%s' % (bioAuth, bio[byidx+23:].lstrip())
                     ndata.append(bio)
                 data[:] = ndata
+                if 'mini biography' in d:
+                    d['mini biography'].append(ndata[0])
+                    continue
             d[sect] = data
         return {'data': d}
 
@@ -655,29 +738,42 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
         cont = subXMLRefs(self._get_search_content('char', name, results))
         name = _findBetween(cont, '<title>', '</title>', maxRes=1)
         res = []
-        if not name: return res
+        if not name:
+            self._mobile_logger.error('no title tag searching character %s',
+                                    name)
+            return res
         nl = name[0].lower()
         if not (nl.startswith('imdb search') or nl.startswith('imdb  search') \
                 or nl.startswith('imdb character')):
-            # XXX: a direct hit!
+            # a direct hit!
             name = _unHtml(name[0]).replace('(Character)', '').strip()
-            pidtag = _getTagsWith(cont, '/character/ch', maxRes=1)
             pid = None
+            pidtag = _getTagsWith(cont, 'rel="canonical"', maxRes=1)
             if pidtag:
-                pid = re_imdbID.findall(pidtag[0])
-            if not (pid and name): return res
-            res[:] = [(str(pid[0]), analyze_name(name, canonical=0))]
+                pid = _findBetween(pidtag[0], '/character/ch', '/', maxRes=1)
+            if not (pid and name):
+                self._mobile_logger.error('no direct hit name/characterID for' \
+                                            ' character %s', name)
+                return res
+            res[:] = [(str(pid[0]), analyze_name(name))]
         else:
-            sects = _findBetween(cont, '<b>Popular Characters</b>', '</table>')
-            sects += _findBetween(cont, '<b>Characters', '</table>')
+            sects = _findBetween(cont, '<b>Popular Characters</b>', '</table>',
+                                maxRes=results*3)
+            sects += _findBetween(cont, '<b>Characters', '</table>',
+                                maxRes=results*3)
             for sect in sects:
-                lis = _findBetween(sect, 'td valign="top">',
+                lis = _findBetween(sect, '<a href="/character/',
                                     ['<small', '</td>', '<br'])
                 for li in lis:
+                    li = '<%s' % li
                     pid = re_imdbID.findall(li)
                     pname = _unHtml(li)
-                    if not (pid and pname): continue
-                    res.append((str(pid[0]), analyze_name(pname, canonical=0)))
+                    if not (pid and pname):
+                        self._mobile_logger.debug('no name/characterID' \
+                                                ' parsing %s searching for' \
+                                                ' character %s', li, name)
+                        continue
+                    res.append((str(pid[0]), analyze_name(pname)))
         return res
 
     def get_character_main(self, characterID):
@@ -693,7 +789,7 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
             if intro:
                 d['introduction'] = intro
         bios = _findBetween(cont, '<div class="display">',
-                            '<div class="history">', maxRes=1)
+                            '<div class="history">')
         if bios:
             bios = _findBetween(bios[0], '<h4>', ('<h4>', '</div>'))
         if bios:
@@ -701,8 +797,10 @@ class IMDbMobileAccessSystem(IMDbHTTPAccessSystem):
                 bio = bio.replace('</h4>', '::')
                 bio = bio.replace('\n', ' ')
                 bio = bio.replace('<br>', '\n')
+                bio = bio.replace('<br/>', '\n')
                 bio = subSGMLRefs(re_unhtmlsub('', bio).strip())
                 bio = bio.replace(' ::', '::').replace(':: ', '::')
+                bio = bio.replace('::', ': ', 1)
                 if bio:
                     d.setdefault('biography', []).append(bio)
         return {'data': d}
